;
; Joypad Tester
; by Bart Trzynadlowski
; December 20, 2005 and January 7, 2006
;

;
; joypad.asm
;
; Reads the 6-button joypad and displays the button status on the screen.
;


;******************************************************************************
; Vector Table
;******************************************************************************

;
; ResetVector:
;
; Execution begins here when the Z80 powers up.
;

.ORG $0
ResetVector:
    di              ; disable interrupts
    ld  sp,$F400    ; set SP to the top of the RAM area
	jp	Start
	
;
; InterruptVector:
; NMIVector:
;
; Do nothing.
;

.ORG $38
InterruptVector:
	reti
	
.ORG $66
NMIVector:
	reti

;******************************************************************************
; Main Program
;******************************************************************************

;
; Start:
;
; Program entry point. 
;

Start:
	;
	; Clear the screen
	;
	ld		hl,$9000	; control reg	
	ld		(hl),4		; settings mode
	ld		a,0
	ld		($8000),a	; disable interrupts and turn screen off
	call	ClearScreen	
	ld		(hl),4		
	ld		a,1
	ld		($8000),a	; turn display on
	
	;
	; Main Loop:
	;
	; - Read joypad
	; - Draw appropriate sprites
	; - Loop forever.
	;
Loop:
	;
	; Read joypad buttons into DE
	;
	call	ReadJoypad
	
	;
	; Draw the correct sprites. Sprite placement is (relative to some base 
	; X,Y coordinates):
	;
	; Up	5,0
	; Down	5,10
	; Left	0,5
	; Right	10,5
	; Start	20,5
	; Mode	25,5
	; A		35,10
	; B		40,10
	; C		45,10
	; X		35,0
	; Y		40,0
	; Z		45,0
	;		
	push	de			; save joypad buttons stored in D
	ld		a,e			; load UDLRBCAS into A
	
	ld		b,5			; set up size of sprite -- same for all!
	ld		c,5
	
	ld		d,100+20
	ld		e,100+5
	ld		hl,spr_start
	call	DrawButtonStatus
	ld		d,100+35
	ld		e,100+10
	ld		hl,spr_a
	call	DrawButtonStatus
	ld		d,100+45
	ld		e,100+10
	ld		hl,spr_c
	call	DrawButtonStatus
	ld		d,100+40
	ld		e,100+10
	ld		hl,spr_b
	call	DrawButtonStatus
	ld		d,100+10
	ld		e,100+5
	ld		hl,spr_right
	call	DrawButtonStatus
	ld		d,100+0
	ld		e,100+5
	ld		hl,spr_left
	call	DrawButtonStatus
	ld		d,100+5
	ld		e,100+10
	ld		hl,spr_down
	call	DrawButtonStatus
	ld		d,100+5
	ld		e,100+0
	ld		hl,spr_up
	call	DrawButtonStatus
	
	pop		de			; retrieve remaining buttons in D (ZYXM)
	ld		a,d			; transfer them to A
	ld		b,5			; size of sprite
	ld		c,5
	ld		d,100+25
	ld		e,100+5
	ld		hl,spr_mode
	call	DrawButtonStatus
	ld		d,100+35
	ld		e,100+0
	ld		hl,spr_x
	call	DrawButtonStatus
	ld		d,100+40
	ld		e,100+0
	ld		hl,spr_y
	call	DrawButtonStatus
	ld		d,100+45
	ld		e,100+0
	ld		hl,spr_z
	call	DrawButtonStatus
	
	jp		Loop
	
;
; ReadJoypad:
;
; Reads all 6 buttons.
;
; Output:
;	DE = 0000 ZYXM UDLR BCAS (M=Mode, S=Start)
;
; Preserves all registers. Video mode ends up as 0.
;
ReadJoypad:	
	push	af	; save used registers
	push	bc
	push	hl
	
	ld		hl,$9000	; Control Register
	ld		bc,$A000	; Joypad Status Register
	
	ld		(hl),8		; MUX=1
	ld		a,(bc)		; A=--UDLRBC
	sla		a
	sla		a
	ld		e,a			; E=UDLRBC00
	ld		(hl),0		; MUX=0
	ld		a,(bc)		; A=--UD00AS
	and		3			; A=000000AS
	or		e			; A=UDLRBCAS
	ld		e,a			; E=UDLRBCAS
	
	ld		(hl),8		; MUX=1
	nop
	ld		(hl),0		; MUX=0
	nop
	
	ld		(hl),8		; MUX=1
	ld		a,(bc)		; A=--ZYXMBC
	srl		a
	srl		a
	and		$F			; A=0000ZYXM
	ld		d,a			; D=0000ZYXM
	
	ld		(hl),0		; MUX=0
	
	pop		hl			; restore registers
	pop		bc
	pop		af
	ret

;
; DrawButtonStatus:
;
; Shifts in the next button from A. Assumes that B, C, D, and E have been set
; up for DrawSprite() appropriately. HL must be set to the button sprite.
;
DrawButtonStatus:	
	rrca						; test button
	jr		nc,button_pressed	; if pressed, use supplied sprite in HL
	ld		hl,spr_blank		; otherwise, blank it out
button_pressed:
	call	DrawSprite
	ret

;
; Sprites (5x5)
;

spr_blank:
	.db 0,0,0,0,0
	.db 0,0,0,0,0
	.db 0,0,0,0,0
	.db 0,0,0,0,0
	.db 0,0,0,0,0
spr_left:
	.db 0,0,7,0,0
	.db 0,7,0,0,0
	.db 7,7,7,7,0
	.db 0,7,0,0,0
	.db 0,0,7,0,0
spr_right:
	.db 0,0,7,0,0
	.db 0,0,0,7,0
	.db 0,7,7,7,7
	.db 0,0,0,7,0
	.db 0,0,7,0,0
spr_up:
	.db 0,0,7,0,0
	.db 0,7,7,7,0
	.db 7,0,7,0,7
	.db 0,0,7,0,0
	.db 0,0,0,0,0
spr_down:
	.db 0,0,0,0,0
	.db 0,0,7,0,0
	.db 7,0,7,0,7
	.db 0,7,7,7,0
	.db 0,0,7,0,0
spr_a:
	.db 0,0,0,0,0
	.db 0,7,7,0,0
	.db 7,0,0,7,0
	.db 7,0,0,7,0
	.db 0,7,7,0,7
spr_b:
	.db 7,0,0,0,0
	.db 7,7,7,0,0
	.db 7,0,0,7,0
	.db 7,0,0,7,0
	.db 7,7,7,0,0
spr_c:
	.db 0,0,0,0,0
	.db 0,7,7,7,0
	.db 7,0,0,0,0
	.db 7,0,0,0,0
	.db 0,7,7,7,0
spr_start:
	.db 0,0,7,7,0
	.db 0,7,0,0,0
	.db 0,7,7,7,0
	.db 0,0,0,7,0
	.db 0,7,7,7,0
spr_x:
	.db 7,0,0,0,7
	.db 0,7,0,7,0
	.db 0,0,7,0,0
	.db 0,7,0,7,0
	.db 7,0,0,0,7
spr_y:
	.db 7,0,0,0,7
	.db 0,7,0,7,0
	.db 0,0,7,0,0
	.db 0,0,7,0,0
	.db 0,0,7,0,0
spr_z:
	.db 7,7,7,7,7
	.db 0,0,0,7,0
	.db 0,0,7,0,0
	.db 0,7,0,0,0
	.db 7,7,7,7,0
spr_mode:
	.db 7,7,0,7,7
	.db 7,0,7,0,7
	.db 7,0,7,0,7
	.db 7,0,0,0,7
	.db 7,0,0,0,7



	

;******************************************************************************
; Video Functions
;******************************************************************************

;
; DrawSprite: 
;
; Draws a sprite on the screen.
;
; Input:
;   B  = X size.
;   C  = Y size.
;	D  = X position.	
;   E  = Y position.
;   HL = Sprite address.
;
; Saves AF, BC, DE, HL.
;
DrawSprite:
    push	af
    push	bc
    push	de
    push	hl
    
    push	hl
    push	bc
    ld		bc,$8000	; data register
    ld		hl,$9000	; command register    
    ld		(hl),2		; set X position high to 0
    ld		a,0
    ld		(bc),a
    ld		(hl),1		; set X position low
    ld		a,d
    ld		(bc),a
	pop		bc
    pop		hl
    
yloop:
	ld		a,3			; set Y position
	ld		($9000),a
	ld		a,e
	ld		($8000),a
	
	ld		a,0			; data write mode
	ld		($9000),a
	push	bc			; save X size

xloop:
	ld		a,(hl)		; fetch pixel
	inc		hl
	ld		($8000),a	; write to framebuffer
	dec		b			; decrement X count and loop
	ld		a,b
	cp		0
	jr		nz,xloop

	pop		bc
	inc		e			; next line in framebuffer
	dec		c			; decrement Y count and loop
	ld		a,c
	cp		0
	jr		nz,yloop
	
	pop		hl
	pop		de
	pop		bc
	pop		af
	ret
;
; ClearScreen:
;
; Clears the screen to black. Saves BC, HL, DE and destroys everything else.
;
ClearScreen:
	push	bc
	push	hl
	push	de
	
	ld		a,1
	ld		($9000),a	; set address X low
	ld		a,0
	ld		($8000),a
	ld		a,2			; set address X high
	ld		($9000),a
	ld		a,0
	ld		($8000),a
	
	ld		b,239	; 240 lines, start clearing at last one
	ld		de,1
ClearScreen_yloop:
	ld		a,3
	ld		($9000),a	; set address Y
	ld		a,b
	ld		($8000),a
	ld		a,0			; data mode
	ld		($9000),a
		
	ld		hl,320	; 320 pixels	
ClearScreen_xloop:	
	ld		($8000),a	; write black pixel
	scf					; clear carry
	ccf
	sbc		hl,de		; X loop
	jr		nz,ClearScreen_xloop
	
	ld		a,b		; if we cleared line 0, quit
	cp		0
	jr		z,ClearScreen_end
	sub		1		; otherwise, keep going
	ld		b,a
	jp		ClearScreen_yloop
ClearScreen_end:
	pop		de
	pop		hl
	pop		bc
	ret	

.END
